%define DOCSTRING
"The `otk` is a python binding for the otk tool kit."
%enddef

%module(docstring=DOCSTRING) otk

%feature("autodoc", "1");

%{
#include "../otk_lib.c"
#include "../gadget_lib.h"
#include "../gadget_lib.c"

extern struct OtkObjectInstance *OtkOuterWindow;
extern float Otk_MouseX, Otk_MouseY;

#define MY_CALLBACK_CODE \
  {	\
  myParameter *my;	\
  my = PyMem_Malloc(sizeof(*my));		\
  my->parameter = parameter;	\
  my->func = func;	\
  Py_XINCREF(func);	\
  Py_XINCREF(parameter);	\
  ret->callback_param = my; \
  return ret;	\
  }

#define MY_PRINT_EXCEPTION	\
      PyObject *o;	\
      o = PyErr_Occurred();	\
      if (o != NULL) {	\
        PyErr_Print();	\
      }

typedef struct myParameter {
  PyObject * parameter;
  PyFunctionObject * func;
} myParameter;

static void myOneArgCallback(void *x) {
  myParameter *my;
  PyObject *ob, *result;
  my = x;
  /* call python function with param */
  if (my->func != NULL) {
    ob = Py_BuildValue("(O)",my->parameter); 
    result = PyEval_CallObject(my->func, ob);
    if (ob != NULL) {
      Py_DECREF(ob);
    }
    if (result != NULL) {
      Py_DECREF(result);
    }
    else {
      PyObject *o;
      o = PyErr_Occurred();
      if (o != NULL) {
        PyErr_Print();
      }
    }
  }
}

static void myTwoArgCallbackString(char *s, void *x) {
  myParameter *my;
  PyObject *ob, *result;
  my = x;
  /* call python function with param */
  if (my->func != NULL) {
    ob = Py_BuildValue("s O",s,my->parameter); 
    result = PyEval_CallObject(my->func, ob);
    if (ob != NULL) {
      Py_DECREF(ob);
    }
    if (result != NULL) {
      Py_DECREF(result);
    }
    else {
      MY_PRINT_EXCEPTION
    }
  }
}

static void myTwoArgCallbackInteger(int state, void *x) {
  myParameter *my;
  PyObject *ob, *result;
  my = x;
  /* call python function with param */
  if (my->func != NULL) {
    ob = Py_BuildValue("i O",state,my->parameter); 
    result = PyEval_CallObject(my->func, ob);
    if (ob != NULL) {
      Py_DECREF(ob);
    }
    if (result != NULL) {
      Py_DECREF(result);
    }
    else {
      /* TODO Through exception */
      MY_PRINT_EXCEPTION
    }
  }
}

static void myTwoArgCallbackFloat(float v, void *x) {
  myParameter *my;
  PyObject *ob, *result;
  my = x;
  /* call python function with param */
  if (my->func != NULL) {
    ob = Py_BuildValue("f O",v,my->parameter); 
    result = PyEval_CallObject(my->func, ob);
    if (ob != NULL) {
      Py_DECREF(ob);
    }
    if (result != NULL) {
      Py_DECREF(result);
    }
    else {
      /* TODO Through exception */
      MY_PRINT_EXCEPTION
    }
  }
}

static PyFunctionObject *MyMouseMove = NULL;

static void myMouseMoveCallback() {
  PyObject *ob, *result;
  /* call python function with param */
  ob = Py_BuildValue("()"); 
  result = PyEval_CallObject(MyMouseMove, ob);
  if (ob != NULL) {
    Py_DECREF(ob);
  }
  if (result != NULL) {
    Py_DECREF(result);
  }
  else {
    /* TODO Through exception */
    MY_PRINT_EXCEPTION
  }
}


static void my_Register_MouseMove_Callback( PyFunctionObject *func )
{
  if (func == NULL) {
    Otk_Register_MouseMove_Callback(NULL);
    Py_XDECREF(func);
  }
  else {
    Otk_Register_MouseMove_Callback(myMouseMoveCallback);
    Py_XINCREF(func);
  }
  MyMouseMove = func;
}

static PyFunctionObject *MyMouseClick = NULL;

static void myMouseClickCallback(int state) {
  PyObject *ob, *result;
  /* call python function with param */
  ob = Py_BuildValue("(i)",state); 
  result = PyEval_CallObject(MyMouseClick, ob);
  if (ob != NULL) {
    Py_DECREF(ob);
  }
  if (result != NULL) {
    Py_DECREF(result);
  }
  else {
    /* TODO Through exception */
    MY_PRINT_EXCEPTION
  }
}

static void my_Register_MouseClick_Callback( PyFunctionObject *func ) {
  if (func == NULL) {
    Otk_Register_MouseClick_Callback(NULL);
    Py_XDECREF(func);
  }
  else {
    Otk_Register_MouseClick_Callback( myMouseClickCallback);
    Py_XINCREF(func);
  }
  MyMouseClick = func;
}

static OtkWidget myMakeSliderHorizontal( OtkWidget container, float left, float top, float horiz_size, PyFunctionObject *func, PyObject *parameter ) 
{
  OtkWidget ret;
  ret =  OtkMakeSliderHorizontal( container, left, top, horiz_size, myTwoArgCallbackFloat, NULL );
  MY_CALLBACK_CODE 
}

static OtkWidget myMakeSliderVertical( OtkWidget container, float left, float top, float vertical_size, PyFunctionObject *func, PyObject *parameter )
{
  OtkWidget ret;
  ret = OtkMakeSliderVertical( container, left, top, vertical_size, myTwoArgCallbackFloat, NULL );
  MY_CALLBACK_CODE 
}

static OtkWidget my_Add_Selection_Item( OtkWidget container, char *text, PyFunctionObject *func, PyObject *parameter ) 
{
  OtkWidget ret;
  ret = Otk_Add_Selection_Item( container, text, myOneArgCallback, NULL );
  MY_CALLBACK_CODE 
}

static OtkWidget my_Add_Menu_Item( OtkWidget container, char *text, PyFunctionObject *func, PyObject *parameter )
{
  OtkWidget ret;
  ret = Otk_Add_Menu_Item( container, text, myOneArgCallback, NULL );
  MY_CALLBACK_CODE 
}

static PyFunctionObject *WindowKillEventCallback = NULL;

static void myWindowKillEventCallback(void *val) {
  PyObject *result=NULL;
  /* call python function with param */
  if (WindowKillEventCallback != NULL) {
    result = PyEval_CallObject(WindowKillEventCallback, NULL);
    if (result != NULL) {
      Py_DECREF(result);
    }
    else {
      /* TODO Through exception */
      MY_PRINT_EXCEPTION 
    }
  }
}

static void my_RegisterWindowKillEventFunction( OtkWidget topobj, PyFunctionObject *func, PyObject *parameter) {
   WindowKillEventCallback = func;
   Otk_RegisterWindowKillEventFunction( topobj, myWindowKillEventCallback, NULL );
}

static PyFunctionObject *MyFilesCallback = NULL;

static void myBrowseFilesCallback(char *fname) 
{
  if (MyFilesCallback != NULL) {
    PyObject *ob, *result;
    ob = Py_BuildValue("(s)",fname); 
    result = PyEval_CallObject(MyFilesCallback, ob);
    if (ob != NULL) {
      Py_DECREF(ob);
    }
    if (result != NULL) {
      Py_DECREF(result);
    }
    else {
      /* TODO Through exception */
      printf("callback name %s\n",fname);
      MY_PRINT_EXCEPTION
    }
  }
}

static OtkWidget my_Browse_Files( char *prompt, int maxlength, char *directory, char *wildcards, char *filename, PyFunctionObject *func )
{
  OtkWidget ret;
  PyObject *parameter = NULL;
  /* Modal dialog only one instance exists at one time */
  MyFilesCallback = func;
  ret = Otk_Browse_Files( prompt, maxlength, directory, wildcards, filename, myBrowseFilesCallback );
  MY_CALLBACK_CODE 
}


static PyFunctionObject *MyBrowseDirsCallback=NULL;

static void myBrowseDirsCallback(char *fname) 
{
  if (MyBrowseDirsCallback != NULL) {
    PyObject *ob, *result;
    ob = Py_BuildValue("(s)",fname); 
    result = PyEval_CallObject(MyBrowseDirsCallback, ob);
    if (ob != NULL) {
      Py_DECREF(ob);
    }
    if (result != NULL) {
      Py_DECREF(result);
    }
    else {
      /* TODO Through exception */
      printf("callback name %s\n",fname);
      MY_PRINT_EXCEPTION
    }
  }
}

static OtkWidget my_Browse_Dirs( char *prompt, int maxlength, char *directory, char *wildcards, char *dirname, void *func) {
  OtkWidget ret;
  PyObject *parameter = NULL;
  MyBrowseDirsCallback = func;
  ret =  Otk_Browse_Dirs( prompt, maxlength, directory, wildcards, dirname, myBrowseDirsCallback );
  MY_CALLBACK_CODE 
}

static OtkWidget myMakeRadioButton( OtkWidget container, float left, float top, float horiz_size, float vert_size, PyFunctionObject *func, PyObject *parameter ) 
{
  OtkWidget ret;
  ret = OtkMakeRadioButton( container, left, top, horiz_size, vert_size, myOneArgCallback , NULL );
  MY_CALLBACK_CODE 
}

static OtkWidget myMakeToggleButton( OtkWidget container, float left, float top, float horiz_size, float vert_size, PyFunctionObject *func, PyObject *parameter ) 
{
  OtkWidget ret;
  ret = OtkMakeToggleButton( container, left, top, horiz_size, vert_size, myTwoArgCallbackInteger , NULL );
  MY_CALLBACK_CODE 
}

static OtkWidget myMakeButton( OtkWidget container, float left, float top, float horiz_size, float vert_size, char *text, PyFunctionObject *func, PyObject *parameter) 
{
  OtkWidget ret;
  ret = OtkMakeButton( container, left, top, horiz_size, vert_size, text, myOneArgCallback, NULL);
  MY_CALLBACK_CODE 
}

static OtkWidget myMakeTextFormBox( OtkWidget container, char *text, int ncols, float x, float y, float horiz_size, float vert_size, PyFunctionObject *func, PyObject *parameter ) 
{
  OtkWidget ret;
  ret = OtkMakeTextFormBox( container, text, ncols, x, y, horiz_size, vert_size, myTwoArgCallbackString, NULL );
  MY_CALLBACK_CODE 
}

static char * OtkGetText(OtkWidget obj, int n) {
  char *p = PyMem_Malloc(n);
  Otk_Get_Text( obj, p, n );
  return p;
}

static void my_Plot_Data( OtkWidget plotbox, int npoints, float *array[2], OtkColor color) {
  Otk_Plot_Data( plotbox, npoints, array[0], array[1], color );
}

static void my_Plot_Data_AutoScaled( OtkWidget plotbox, int npoints, float *array[2], OtkColor dcolor, OtkColor tcolor, OtkColor gcolor, int xgrids, int ygrids ) {
  Otk_Plot_Data_AutoScaled( plotbox, npoints, array[0], array[1], dcolor, tcolor, gcolor, xgrids, ygrids );
}

%}

%feature("autodoc", "1");
%immutable;
struct OtkTabbedPanel
{
  //OtkWidget top;
  int num;
  //char **names;
  //OtkWidget panel_top;
  //OtkWidget *panels;
  //OtkWidget *buttons;
  //OtkTabbedPanelSelect *selects;
  //int selection;
  //float panel_height;
  //float button_height;
};
%mutable;

%extend OtkTabbedPanel {
  OtkWidget getPanel(int num) {
    if (num < $self->num) {
      return $self->panels[num];
    }
    else {
      return NULL;
    }
  }
};

%extend OtkObjectInstance {
 ~OtkObjectInstance() {
     if ($this->callback_param != NULL) {
       myParameter *my = $this->callback_param;
       Py_XDECREF(my->func);
       PyMem_Free(my);
       Py_XDECREF(my->parameter);
     }
  }
};

%immutable;
struct OtkColor_record
{
  float r, g, b;
};

%mutable;


%typedef struct OtkTabbedPanel OtkTabbedPanel;
%typedef struct OtkObjectInstance * OtkWidget;
%typedef struct OtkColor_record OtkColor;

%immutable;
extern struct OtkObjectInstance *OtkOuterWindow;
extern float Otk_MouseX, Otk_MouseY;
%mutable;


#define Otk_Flat     0                          /* Panel Types. */
#define Otk_Raised   1
#define Otk_Recessed 2
#define Otk_Invisible 10
#define Otk_ImagePanel 20

#define Otk_SC_Panel      1                     /* Super-Classes. */
#define Otk_SC_TextLabel  2
#define Otk_SC_Button     3
#define Otk_SC_FormBox    4
#define Otk_SC_Line       5
#define Otk_SC_Box        500
#define Otk_SC_hSlider    6
#define Otk_SC_vSlider    7
#define Otk_SC_textscrollbar 8
#define Otk_SC_Window       10
#define Otk_SC_WindowPane 	    11
#define Otk_SC_Menu_DropDown_Button 12
#define Otk_SC_Menu_Item            14
#define Otk_SC_Menu_Submenu         15
#define Otk_SC_RadioButton          20
#define Otk_SC_ToggleButton         30
#define Otk_SC_Select_List          40
#define Otk_SC_Select_List_Item     41
#define Otk_SC_Select_List_SubList  42
#define Otk_SC_TabHandle            300

#define Otk_class_panel         1               /* Base Classes. */
#define Otk_class_text          2
#define Otk_class_button        3
#define Otk_class_line          5
#define Otk_class_box           500
#define Otk_class_radiobutton1 20
#define Otk_class_radiobutton2 21
#define Otk_class_togglebutton 30
#define Otk_class_triangle    100
#define Otk_class_disk        200
#define Otk_class_circle      250
#define Otk_class_userdefd   1000
#define Otk_class_other      2000

#define Otk_subtype_plane            0          /* Sub-Types. */
#define Otk_subtype_raised           1
#define Otk_subtype_recessed         2
#define Otk_subtype_moveable         3
#define Otk_subtype_recessed_diamond 4
#define Otk_subtype_raised_diamond   5
#define Otk_hover_mask        1
#define Otk_MouseCursor_Arrow 0
#define Otk_MouseCursor_Cross 1
#define Otk_MouseCursor_Hand  2
#define Otk_MouseCursor_Wait  3

#define Otk_Font_Vect 0         /* Very basic vector built-in font. */
#define Otk_Font_Helv 1         /* Second and nicer built-in font. */
#define Otk_FontDefault Otk_Font_Helv

#define Otk_FontSpacing_Mono 0
#define Otk_FontSpacing_Proportional 1
#define Otk_FontSpacingDefault Otk_FontSpacing_Mono

void Otk_Set_Default_Border_Thickness( float thickness );
void Otk_Set_Object_Border_Thickness( OtkWidget obj, float thickness );
OtkColor OtkSetColor( float r, float g, float b );

%feature("docstring") OtkMakePanel "
     This function makes a rectangular panel, either flat, raised, or recessed, within the
     parent window or panel.  The concept of the panel, also called a container widget,
     is a very important aspect of Otk.  All other widgets and sub-panels must be placed on a 
     container/panel.  The OtkOuterWindow is the first container/panel, and is provided by default.
     Placing sub-panels on a panel provides a method of dividing-up the parent panel.

     Returns OtkWidget.

     Container - is the parent container-panel, onto which the new panel is placed, and to whose
       frame the coordinates will be relative.

     Panel types currently include:
        Otk_Flat = Plan flat rectangle.
        Otk_Raised = Raised rectangle.
        Otk_Recessed = Recessed Rectangle.
       (In the future, more panel types could be added.)

     Color is an OtkColor record.

     The left, top, coordinates are relative to the parent window/container origin.
     All coordinates are in percentage of outer window size, where the upper left corner is (0.0,0.0)
     and the lower right corner is (100.0,100.0).

     The following related functions set additional attributes or change default settings:

     Otk_Set_Object_Border_Thickness( OtkWidget panel, float thickness )
     Otk_Set_Default_Border_Thickness( float thickness )
     OtkResizePanel( OtkWidget panel, float left, float top, float horiz_size, float vert_size )
     Otk_Tabbed_Panel_New( parent, num, **names, color, left, top, width, height, button_height )

     See Panel Functions for more on panel-related functions.
";

OtkWidget OtkMakePanel( OtkWidget container, int panel_type, OtkColor panel_color, float left, float top, float horiz_size, float vert_size );
void OtkResizePanel( OtkWidget panel, float left, float top, float horiz_size, float vert_size );
OtkWidget OtkMakeImagePanel( OtkWidget container, char *file_name, float left, float top, float horiz_size, float vert_size );
%feature("docstring") Otk_Dispose_Image "
        Removes all resources associated with the named image, from the
        graphics-card, when an image is no longer needed.  This is to be
        used after an image was loaded via OtkMakeImagePanel, and/or related
        image functions, and later the object is removed and it is known
        the image is no longer needed.  (Normally, one copy of each image
        is retained in graphics memory that can be referenced by many
        object instances, and for rapid re-use.)
"
void Otk_Dispose_Image( char *file_name );

%feature("docstring") Otk_Add_Line "
   Often it is useful to add lines or separators to a GUI panel.  General purpose lines can be
   added with the Otk_Add_Line function.  Specify the window/container, line color, thickness, 
   and the coordinates to draw the line from (x1,y1) and to (x2,y2).  The coordinates are in 
   percentages of the container-window.  Boxes can be drawn as a set of four lines.
"
OtkWidget Otk_Add_Line( OtkWidget container, OtkColor tmpcolor, float thickness, float x1, float y1, float x2, float y2 );
%feature("docstring") Otk_Add_BoundingBox "
   Makes a rectangular separating boundary outline, whose upper left corner is (x1,y1) and lower
   right corner is (x2,y2).
"
OtkWidget Otk_Add_BoundingBox( OtkWidget container, OtkColor tmpcolor, float thickness, float x1, float y1, float x2, float y2 );
void Otk_Set_Line_Thickness( OtkWidget tmpobj, float thickness );

%feature("docstring") OtkMakeTextLabel "
This function places text onto a panel or window.

    Returns OtkWidget.

    Panel - is the container-panel, or parent widget, on which the label is attached/placed.
        And to whose frame the coordinates will be relative!
    Text - is the character string array to make the label out of.
    Fontcolor - an OtkColor record.
    fontsize - A float scaling factor.  1.0 is normal size, with each character being 12.0% high
        by 8.0% wide in Otk coordinates.
    fontweight - 1.0 is normal. 2.0 is bold (double-thick).
    x, y - Coordinates of the origin of the label.  Upper left edge relative to containing panel's 
        upper left origin.

    The following related functions set additional attributes or change default settings:
        Otk_Modify_Text( OtkWidget label, char *text ) 
        Otk_Modify_Text_Aspect( OtkWidget label, float aspect ) 
        Otk_Modify_Text_Scale( OtkWidget label, float scale ) 
        Otk_Modify_Text_Color( OtkWidget label, OtkColor text_color ) 
        Otk_Modify_Text_Slant( OtkWidget label, float slant ) 
        Otk_Modify_Text_Thickness( OtkWidget label, float thickness ) 
        Otk_Modify_Text_Position( OtkWidget label, float x, float y ) 
            Otk_Set_Text_Aspect( float aspect ) 
        Otk_Get_Character_Size( OtkWidget label, float *width, float *height ) 
        Otk_Get_Text_Size( OtkWidget label, float *width, float *height ) 
        Otk_FitTextInPanel( OtkWidget label ) 
"

OtkWidget OtkMakeTextLabel( OtkWidget container, char *text, OtkColor text_color, float scale, float thickness, float x, float y );
void Otk_Modify_Text_Slant( OtkWidget tmpobj, float slant );
void Otk_Modify_Text_Color( OtkWidget tmpobj, OtkColor text_color );
void Otk_Modify_Text_Thickness( OtkWidget tmpobj, float thickness );
void Otk_Modify_Text_Scale( OtkWidget tmpobj, float scale );
void Otk_Modify_Text_Aspect( OtkWidget tmpobj, float aspect );
void Otk_Set_Text_Aspect( float aspect );
void Otk_Modify_Text_Position( OtkWidget tmpobj, float x, float y );
void Otk_Get_Text_Size( OtkWidget tmpobj, float *width, float *height );
void Otk_Get_Character_Size( OtkWidget tmpobj, float *width, float *height );
void Otk_Modify_Text( OtkWidget tmpobj, char *text );
void Otk_FitTextInPanel( OtkWidget txtobj );

%feature("docstring") myMakeButton "
     This function makes a push-button that will call the named user's function when pressed.

     Returns OtkWidget.

        Panel - is the container-panel, or parent widget, on which the button is attached/placed.
                And to whose frame the coordinates will be relative!

        x, y - Coordinates of the button's upper-left corner on the containing panel (in %).
                Relative to containing panel's upper left origin.

     horiz_size, vert_size - Horizontal and vertical size of the button, in % of the 
        containing window's size.

        Text - is the character string array to put onto the button.

     CallBack - Function which gets called back when button is pressed.
     CallParam - Parameter passed to callback function.
"
%typemap(in) PyFunctionObject *func {
/*  1 - $1, input - $input */
  if (!PyFunction_Check($input)) {
    /* TODO through execption */
    printf("$symname $1 $1_name is not a function\n");
  }
  $1 = $input;
}
%rename(OtkMakeButton) myMakeButton;
OtkWidget myMakeButton( OtkWidget container, float left, float top, float horiz_size, float vert_size, char *text, PyFunctionObject *func = NULL, PyObject *parameter = NULL);
void Otk_Set_Default_Button_Color( float r, float g, float b );
void Otk_Set_Button_Color( OtkWidget button, OtkColor panel_color );
void Otk_Set_Panel_Color( OtkWidget container, OtkColor panel_color );
void Otk_Set_Default_Button_BorderThickness( float x );
void Otk_Set_Button_BorderThickness( OtkWidget button, float x );
void Otk_Set_Button_Outline_Style( int style );
void Otk_Set_Button_State( OtkWidget button, int state );
int Otk_Get_Button_State( OtkWidget button );
%feature("docstring") myMakeToggleButton "
    This function makes a simple toggle-button on the panel.  When pressed, it toggles it's state,
    and calls the named user function while passing the given parameter.  The parameter, such as an
    integer, can be used to identify which button was pressed.

    Returns OtkWidget handle.
    Where:
        container - is the container-panel, or parent widget, on which the button is attached/placed.
                And to whose frame the coordinates will be relative!
        x, y - Coordinates of the button's upper-left corner on the containing panel (in %).
                Relative to containing panel's upper left origin.
    horiz_size, vert_size - The horizontal and vertical size of the button in percentage
        of the containing window.
    callback - User function called when toggled.
    parameter - parameter passed to user function.

    The following are related functions:

    void Otk_Set_Button_State( OtkWidget button, int state )

        Causes named button to assume the specified state of:
            0 = not toggled, not depressed.
            1 = toggled on, depressed.

    int Otk_Get_Button_State( OtkWidget button)

        Returns the named button's state as 0=not-depressed, or 1=depressed.
"
%rename(OtkMakeToggleButton) myMakeToggleButton;
OtkWidget myMakeToggleButton( OtkWidget container, float left, float top, float horiz_size, float vert_size, PyFunctionObject *func = NULL, PyObject *parameter  = NULL);
%feature("docstring") myMakeRadioButton "
    This function places a radio-button onto the panel.  The first radio-button of a set must name the
    parent-panel/window as the container.  The remaining radio-buttons in a set must name one of the
    other buttons in the set as their parent-container.  When pressed, the pressed radio button is
    set, and the others are un-set.  Only one radio-button in a set has the depressed state at a time.
    The named user function is called when pressed, and the given parameter value is passed.
    The parameter, such as an integer, can be used to identify which button was pressed.

    Returns OtkWidget handle.
    Where:
        container - is the container-panel, or parent widget, on which the button is attached/placed.
                And to whose frame the coordinates will be relative!
        x, y - Coordinates of the button's upper-left corner on the containing panel (in %).
                Relative to containing panel's upper left origin.
    horiz_size, vert_size - The horizontal and vertical size of the button in percentage
        of the containing window.
    callback - User function called when toggled.
    parameter - parameter passed to user function.

    The following are related functions:    
    
    void Otk_SetRadioButton( OtkWidget radiobutton )

        Causes named button to be depressed, while releasing all others in the set.

    int Otk_Get_Button_State( OtkWidget button)

        Returns the named button's state as 0=not-depressed, or 1=depressed.
"
%rename(OtkMakeRadioButton) myMakeRadioButton;
OtkWidget myMakeRadioButton( OtkWidget container, float left, float top, float horiz_size, float vert_size, PyFunctionObject *func = NULL, PyObject *parameter = NULL );
void Otk_SetRadioButton( OtkWidget topobj );
void Otk_Set_Button_Icon( OtkWidget button, struct Otk_image *image_ptr );
void Otk_Set_Button_Icon_File( OtkWidget button, char *file_name );

%feature("docstring") OtkMakeWindow "
  This function makes a floating sub-window which can be closed by the user.
  Widgets can then be added to this window, like any other container-window.
"
OtkWidget OtkMakeWindow( int panel_type, OtkColor tab_color, OtkColor panel_color,
                                 float left, float top, float horiz_size, float vert_size );
%feature("docstring") OtkSetWindowTitle "
  This function sets a sub-window's title.  It places text in the window title-bar.
"
void OtkSetWindowTitle( OtkWidget window, OtkColor text_color, char *title );
%feature("docstring") Otk_RegisterWindowKillEventFunction "
  This function registers your own function to be called if the user closes the respective window
  by the X in the window's handle..
"
%rename(Otk_RegisterWindowKillEventFunction) my_RegisterWindowKillEventFunction;
void my_RegisterWindowKillEventFunction( OtkWidget topobj, PyFunctionObject *func = NULL , PyObject *parameter = NULL );

%typemap(check) PyFunctionObject *func {
/*  1 - $1, input - $input */
}
%typemap(check) PyObject *parameter {
/*  1 - $1, input - $input */
}
%feature("docstring") myMakeTextFormBox "
   This function places a text fill-in box on the panel.  Initial text can be placed in the box, or
   it can be initialized blank ("").  Your own call-back function is called when <Enter> is
   pressed.  The text can also be read from the box by the Otk_Get_Text function below.

    Returns OtkWidget handle.

    Where:
        container - is the container-panel, or parent widget, on which the form-box is attached/placed.
                And to whose frame the coordinates will be relative!
    text - any initial text to place in the fill-in box.  Can be empty string, "".
    ncols - number of characters to fit in the box.  (Note that the width of the box is specified
        by the horiz_size parameters, so the ncols really sets the horizontal size of the 
        characters to fit in the box of that size.)
        x, y - Location of the fill-in box's upper-left corner on the containing panel (in %).
                Relative to containing panel's upper left origin.
    horiz_size, vert_size - The horizontal and vertical size of the fill-in box in percentage
        of the containing window.
    callback - Your function to call when <enter> is pressed.  
    parameter - arbitrary parameter to pass to your callback function.

    The following are related functions:   

        void Otk_Modify_Text( OtkWidget textbox, char *text ) 

        Places the specified text in the form box.
    
        void *Otk_Get_Text( OtkWidget textbox, char *text, int n ) 

        Returns any text from the specified form-box into the character-array in the
        second parameter.  The third parameter should be specified as the maximum length of
        the character array for safety.
"
%rename(OtkMakeTextFormBox) myMakeTextFormBox;
OtkWidget myMakeTextFormBox( OtkWidget container, char *text, int ncols, float x, float y, float horiz_size, float vert_size, PyFunctionObject *func = NULL, PyObject *parameter = NULL);

%newobject OtkGetText;
char * OtkGetText(OtkWidget obj, int n);
/*void Otk_Get_Text( OtkWidget tmpobj, char *text, int n );*/
%feature("docstring") OtkMakeTextEditBox "
    This function places a multi-row text-edit box on the panel.  Initial text can be placed in the box, or
    it can be initialized blank ('').  The text can be read from the box by the Otk_Get_Text function below.
    Much like the simple text-box above, but enables more than one row, and has no callback function, since
    <enter> is legal character to break lines in the edit-box.  Read any entered text with the 
    Otk_Get_Text (below), perhaps initiated by an 'ok' button callback..

    Returns OtkWidget handle.

    Where:
        container - is the container-panel, or parent widget, on which the edit-box is attached/placed.
                And to whose frame the coordinates will be relative!
    text - any initial text to place in the edit-box.  Can be empty string, ''.
    ncols - number of character columns to fit in the box.  (Note that the size of the box is 
        specified by the horiz_size and vert_size parameters, so ncols and nrows really set
        the horizontal and vertical size of the characters to fit in the box.)
    nrows - number of character rows to fit in the box.  
        x, y - Location of the edit-box's upper-left corner on the containing panel (in %).
                Relative to containing panel's upper left origin.
    horiz_size, vert_size - The horizontal and vertical size of the edit-box in percentage
        of the containing window.

    The following are related functions:

        OtkAddTextScrollbar( OtkWidget container, float width )

        Otk_Get_Text( OtkWidget textbox, char *text, int n )
"
OtkWidget OtkMakeTextEditBox( OtkWidget container, char *text, int nrows, int ncols,
                                float left, float top, float horiz_size, float vert_size );
void OtkAddTextScrollbar( OtkWidget container, float width );
void Otk_Right_Justify_Formbox( OtkWidget formbox );

%feature("docstring") my_Browse_Files "
This is a higher level convenience widget that is useful for helping the user find files to open, 
save-as, import, or etc..  You can indicate the intended operation to the user through the prompt
argument.  The Otk_Browse_Files widget allows navigation of the file system by showing the
files in the directory, and by going up and down into other directories.  It allows filtering the
displayed files.  Defaults can be supplied for the file name, directory, and filter (wildcards), 
which the user can also change.  The Otk_Browse_Files routine will call the user-supplied
call-back routine when a file is selected.  It passes the selected file's name as a character
string into the call-back function, where you can open the file for reading/writing, or whatever.  

Where:
 prompt - Input (can be constant literal string).  The 'title' string or purpose
     of the browser window.  For example:  'Open' 'Save-As' 'Import' etc..
 maxlength - The minimum array size of the next three character string parameters, 
    (directory, wildcard, filename).  If any strings to be returned are longer than or equal
    to maxlength, then the returned string is truncated to maxlength-1 characters for safety.
 directory - Input/Output.  Must point to an allocated[*] character string array variable
    of size maxlength or greater.  This specifies the starting directory for browsing.
    Often set to '.' (local directory) initially.  If user browses to other directories,
    then last visited directory path is copied to this string.  Path may be relative or absolute.
 filetype_filter - Input/Output.  Must point to an allocated[*] character string array
    variable of size maxlength or greater.  This specifies the type of files to filter and
    list, much like file-wildcards, such as *.doc .  For example, '*.txt *.dat', 
    or for everything '*'.  If the user changes the file-type setting, the new setting is
    returned in this string.
 filename - Input/Output.  Must point to an allocated[*] character string array variable of
        size maxlength or greater.  This specifies any initial file-name to place in the
    file-name box, if any.  Often, an empty file-name string is passed in.
    The selected file name is copied into this string.  The whole purpose of this
    widget is to select the file-name, so this is the primary output of the routine.
    At most maxlength-1 characters will be copied into the file-name array.
 callback - The user's callback function.  Function will called when file is selected.
    The selected file is passed as the argument.

For complete example, see: File Browser Example.
"
%rename(Otk_Browse_Files) my_Browse_Files;
OtkWidget my_Browse_Files( char *prompt, int maxlength, char *directory, char *wildcards, char *filename, PyFunctionObject *func = NULL);

%feature("docstring") my_Browse_Dirs "
Sometimes your application needs to locate a directory; not a particular file.
In this case, is the Otk_Browse_Dirs function, instead of Otk_Browse_Files.

This function displays only directories. Users can navigate directories by double-clicking them.
The final directory is selected by clicking the OK button.
"
%rename(Otk_Browse_Dirs) my_Browse_Dirs;
OtkWidget my_Browse_Dirs( char *prompt, int maxlength, char *directory, char *wildcards, char *dirname, PyFunctionObject *func = NULL );
%feature("docstring") Otk_fb_add_optional_dirfile "
You can provide a pull-down menu to the right of the file-name entry, for the purpose of
suggesting alternate, or recently visited, directory(s) or files
"
void Otk_fb_add_optional_dirfile( char *dir_or_file );  
%feature("docstring") Otk_fb_clear_optional_dirfiles "
All entries made by this function will be available on all future calls to the
file-browser, until cleared by this function.
"
void Otk_fb_clear_optional_dirfiles(); /* Clears previous alternate directory entries. */
%feature("docstring") Otk_fbrowse_size_hint "
The file and directory-browser gadgets attempt to display themselves at a reasonable size and
aspect ratio, depending on the application window's size.  For various reasons, you may wish
to have the broswer be a different size, aspect ratio, or screen position.  You can control
these things with the Otk_fbrowse_size_hint function.

The coordinates are all in percent of the outer window.
"
void Otk_fbrowse_size_hint( float width, float height, float horiz_center_offset,  float vert_center_offset );

int Otk_handle_key_input( int ks );
void Otk_handle_key_release( int ks );
int Otk_UpdateMouse( int x, int y, int state );
int Otk_handle_mouse_move( int MouseDx, int MouseDy );

%feature("docstring") Otk_Make_Menu "
   This function creates a menu pull-down.  This forms the top-level entry-point to a pull-down
   menu.  Items are then added to the menu with the Otk_Add_Menu_Item and/or Otk_Add_SubMenu functions
   described below.
"
OtkWidget Otk_Make_Menu( OtkWidget container, float left, float top, float horiz_size, float vert_size, char *text );
%feature("docstring") my_Add_Menu_Item "
   This function adds a menu item to a pull-down menu.
"
%rename(Otk_Add_Menu_Item) my_Add_Menu_Item;
OtkWidget my_Add_Menu_Item( OtkWidget container, char *text, PyFunctionObject *func = NULL, PyObject *parameter = NULL );
%feature("docstring") Otk_Add_SubMenu "
   This function creates a sub-menu pull-down on an existing menu.
"
OtkWidget Otk_Add_SubMenu( OtkWidget container, char *text );
%feature("docstring") Otk_Set_Menu_Selectable "
   This function deativates (0) or activates (1) a menu-item. 
"
void Otk_Set_Menu_Selectable( OtkWidget tmpobj, int on_notoff );

%feature("docstring") Otk_Make_Selection_List "
   This function creates a scrollable selection-list container at the specified location and size.
   Use Otk_Add_Selection_Item to add items to the list.  If more than rows items are added,
   a scroll-bar is automatically added to the list.
"
OtkWidget Otk_Make_Selection_List( OtkWidget container, int rows, int cols,
                                   float left, float top, float horiz_size, float vert_size );
void Otk_Frame_Selection_List( OtkWidget container );
%feature("docstring") my_Add_Selection_Item "
   This function adds the specified text string to the referenced selection-list.  If selected by the
   user, the named callback function will be called and the specified parameter will be passed.
   In actual usage, often, all entries are assigned a common callback function with parameters assigned
   sequentially to identify the selected item in the callback.
"
%rename(Otk_Add_Selection_Item) my_Add_Selection_Item;
OtkWidget my_Add_Selection_Item( OtkWidget container, char *text, PyFunctionObject *func = NULL, PyObject *parameter = NULL );
%feature("docstring") Otk_Coordinate_Selection_Lists "
   This function links multiple selection lists together, so that all are controlled
   by a single scroll-bar, and all scroll together.  It is a convenient way to create
   multiple-column scrolling selection lists.  This function is to be called after the list
   widgets have been created with Otk_Make_Selection_List, but before any items have
   been added to the lists.  One list is designated as the master-list, since it will
   have the scroll-bar.  Usually, all lists in a group are created next to each other,
   at equal height and size.  The right-most list is usually designated as the master,
   since the scroll-bar attaches to the right side, when there are sufficient items in
   the list.  It is assumed that all lists in such a group will always hold an equal
   number of items.  See example application: multiple_selection_lists.py.
"
void Otk_Coordinate_Selection_Lists( OtkWidget master_list, OtkWidget subordinate_list );

%feature("docstring") Otk_MakeDisk "
Places a solid disk of the specified color and radius centered at (x,y).
"
OtkWidget Otk_MakeDisk( OtkWidget container, float x, float y, float radius, OtkColor disk_color );
%feature("docstring") Otk_MakeCircle "
  Places an empty circular ring of the specified color, radius, and line-thickness, centered at (x,y).
  Normal thickness is 1.0, while 2.0 is double thickness, etc..
"
OtkWidget Otk_MakeCircle( OtkWidget container, float x, float y, float radius, OtkColor circ_color, float thickness );


%feature("docstring") myMakeSliderVertical "
    This function places a vertical slider onto the named panel.  See OtkMakeSliderHorizontal above.
"
%rename(OtkMakeSliderVertical) myMakeSliderVertical;
OtkWidget myMakeSliderVertical( OtkWidget container, float left, float top, float vertical_size, PyFunctionObject *func = NULL, PyObject *parameter  = NULL);
%feature("docstring") myMakeSliderHorizontal "
    This function places a horizontal slider onto the named panel.  The named user function
    is called when the slider is moved.  The new slider position is passed as the first value
    in the range 0.0 to 1.0.  The given parameter is passed as the second value.  The parameter
    can be used for a variety of purposes, and it can be a pointer to a larger data structure.
    One usage of the parameter is to uniquely identify which slider moved, when a single
    call-back routine is used to handle multiple sliders.
"
%rename(OtkMakeSliderHorizontal) myMakeSliderHorizontal;
OtkWidget myMakeSliderHorizontal( OtkWidget container, float left, float top, float horiz_size, PyFunctionObject *func = NULL, PyObject *parameter  = NULL);
%feature("docstring") Otk_SetSlider "
    This function places the slider knob at the specified position, from 0.0 to 1.0.
    For vertical sliders, 0.0 is the bottom, and 1.0 is the top.  For horizontal sliders,
    0.0 is the left side, and 1.0 is the right side.  These values also correspond to
    the values returned from the slider.  

    This function can also set the size of the slider knob.  The default knob size is 1.0.
"
void Otk_SetSlider( OtkWidget slider, float position, float sz );
void Otk_Set_Default_Slider_Width( float x );

%feature("docstring") Otk_Register_Hover_Box "
   Hover-boxes, sometimes called Tool-Tips, are useful to provide hints about buttons 
   or other controls.  They pop-up a small text-message when the mouse hovers over the given
   item for a prolonged period.

   This function associates the given widget, usually a button or menu, with the text message.
   The message will pop-up when the mouse hovers over the given widget.
"
void Otk_Register_Hover_Box( OtkWidget obj, char *text );
%feature("docstring") Otk_Register_Hover_Box "
   This function un-registers the hover-message from a widget.
"
void Otk_Unregister_Hover_Box( OtkWidget obj );
void Otk_Set_HoverBox_Style( OtkColor box_color, OtkColor text_color, float text_size );
%feature("docstring") Otk_Enable_Hoverboxes "
   This function enables all hover-boxes.
   Hover-boxes are initially enabled by default.
"
void Otk_Enable_Hoverboxes();
%feature("docstring") Otk_Disable_Hoverboxes "
   This function disable all hover-boxes.
   Hover-boxes are initially enabled by default.
"
void Otk_Disable_Hoverboxes();

%typemap(in) (int argc, char **argv) {
  int i;
  if (!PyList_Check($input)) {
    PyErr_SetString(PyExc_ValueError, "Expecting a list");
    return NULL;
  }
  $1 = PyList_Size($input);
  $2 = (char **) malloc($1*sizeof(char *));
  for (i = 0; i < $1; i++) {
    PyObject *s = PyList_GetItem($input,i);
    if (!PyString_Check(s)) {
        free($2);
        PyErr_SetString(PyExc_ValueError, "List items must be strings");
        return NULL;
    }
    $2[i] = PyString_AsString(s);
  }
}
%typemap(freearg) (int argc, char **argv) {
   if ($2) free($2);
}
%feature("docstring") OtkInitWindow "
  This function is one of the most important.  It produces the initial outer window,
  called OtkOuterWindow,  You specify the initial size of the window in pixels
  for width and height.  You can pass command-line arguments as argc and argv.
  It must be called first, before any other Otk calls.  OtkInitWindow is intended for
  stand-alone applications.
"
void OtkInitWindow( int WinWidth, int WinHeight, int argc = 0, char **argv = NULL);
%feature("docstring") OtkMakeOuterWindow "
  This function is the alternative to OtkInitWindow above, for applications that are
  part of another OpenGL application which already opened the outer-window.
  It initializes the Otk global variables and establishes the OtkOuterWindow widget 
  within the existing window.  It must be called first, before any other Otk calls.
  Often used with OtkUpdateCheck().
"
void OtkMakeOuterWindow();
void OtkUpdateWindow( int WinWidth, int WinHeight );

%feature("docstring") OtkUpdateCheck "
        Check-for and handle any screen updates or user interaction events.
        Normally, this is handled by the OtkMainLoop() function.
        However, some applications may either be busy for a long while, or do not
        wish to have Otk be the main routine.  (OtkMainLoop never returns, it
        just calls other routines.)  In such situations, call OtkUpdateCheck() 
        periodically.  It allows your application to respond to user interactions 
        or screen updates.  OtkUpdateCheck simply returns if there are no pending
        events, or handles them and returns, but never blocks.
"
void OtkUpdateCheck();
%feature("docstring") Otk_RemoveObject "
Removes the referenced object, and it's children.
"
OtkWidget Otk_RemoveObject( OtkWidget objt );
void Otk_ClearAll();
void Set_PermanentObjs();
void OtkDrawObjectTree();
void Otk_ReDraw_Display();

%typemap(in) (int num, char **names) {
  int i;
  if (!PyList_Check($input)) {
    PyErr_SetString(PyExc_ValueError, "Expecting a list");
    return NULL;
  }
  $1 = PyList_Size($input);
  $2 = (char **) malloc(($1+1)*sizeof(char *));
  for (i = 0; i < $1; i++) {
    PyObject *s = PyList_GetItem($input,i);
    if (!PyString_Check(s)) {
        free($2);
        PyErr_SetString(PyExc_ValueError, "List items must be strings");
        return NULL;
    }
    $2[i] = PyString_AsString(s);
  }
  $2[i] = 0;
}
%typemap(freearg) (int num, char **names) {
   if ($2) free($2);
}
OtkTabbedPanel *Otk_Tabbed_Panel_New( OtkWidget parent, int num, char **names, OtkColor color, float left, float top, float width, float height, float button_height );

%feature("docstring") Otk_Snapshot "
        Captures an image snapshot of the current screen window to the specified filename.
        It saves it as a PPM color image, which can be converted to other formats by
        a variety of means, such as the NetPBM utilities.
"
void Otk_Snapshot( char *fname );
%feature("docstring") Otk_SnapShot_Size "
        Sets snap-shot size: 1=full-size, 2=half-size.  Half-size images require less time
        to save, and are automatically anti-aliased for improved image appearance.
        Useful for rapid repeated saves, such as capturing making frames.
"
int Otk_SnapShot_Size( int sz );
%feature("docstring") Otk_MakeMovie "
        A value of 1, begins capturing frames at a regular rate.  The snapshot frames are
        sequentially numbered, and can be combined into MPEG or AVI movies, via
        tools such as mencoder or ppmtompeg.
"
void Otk_MakeMovie( int on_notoff );

int Otk_Get_Shift_Key();
int Otk_Get_Control_Key();
int Otk_Get_MouseButton_Clicked();   /* Returns the number of last mouse-button clicked, 1-3. */
%feature("docstring") Otk_Set_Mouse_Cursor "
    Sometimes it is useful to change the mouse-cursor to a different shape.

    Where the cursor_style can be one of the following:
    Otk_MouseCursor_Arrow   - Default pointer.
    Otk_MouseCursor_Cross   - Useful when ready to mark a location.
    Otk_MouseCursor_Hand    - Useful when in grabbing mode.
    Otk_MouseCursor_Wait    - Useful while waiting for something (ex. Hour-glass)
"
void Otk_Set_Mouse_Cursor( int cursor_style );
/* Otk Internal or Special Functions. */
void Otk_object_attach( OtkWidget parent, OtkWidget child );
void Otk_object_attach_at_end( OtkWidget parent, OtkWidget child );
void Otk_object_attach_hidden( OtkWidget parent, OtkWidget child );
void Otk_object_attach_hidden_at_front( OtkWidget parent, OtkWidget child );
void Otk_object_detach( OtkWidget child );
void Otk_object_detach_hidden( OtkWidget child );
void Otk_object_detach_any( OtkWidget child );
void Otk_object_correct_position( OtkWidget obj, int descend );
void Otk_calculate_object_position( OtkWidget container, OtkWidget obj );
void Otk_position_object( OtkWidget objpt, int absolute, float x, float y );
void Otk_move_object( OtkWidget objpt, int absolute, float x, float y );
void Otk_next_word( char *line, char *word, char *delim );
double otk_report_time();
void Otk_Start_Capturing_TestFile();
void Otk_Close_TestFile();
void Otk_Apply_Test_Stimuli( char *filename );
%feature("docstring") my_Register_MouseClick_Callback "
   The state value of the MouseClick_Callback function passes in 0 when the left 
   mouse-button is down, and 1 when the mouse-button is up.  When your callback
   function is called, the value of the global variables,
   cvar.Otk_MouseX and cvar.Otk_MouseY, hold the mouse position in OTK coordinates,
   which are the percentage of the window from 0.0 to 100.0 horizontally and vertically,
   respectively.
"
%rename(Otk_Register_MouseClick_Callback) my_Register_MouseClick_Callback;
void my_Register_MouseClick_Callback( PyFunctionObject *func  = NULL);
%feature("docstring") my_Register_MouseMove_Callback "
   When your callback function is called, the value of the global variables,
   cvar.Otk_MouseX and cvar.Otk_MouseY, hold the mouse position in OTK coordinates,
   which are the percentage of the window from 0.0 to 100.0 horizontally and vertically,
   respectively.
"
%rename(Otk_Register_MouseMove_Callback) my_Register_MouseMove_Callback;
void my_Register_MouseMove_Callback( PyFunctionObject *func  = NULL);
%feature("docstring") Otk_Set_Window_Name "
  This function sets the outer-window's name in the window's title-bar.
  It must be called before calling: OtkInitWindow.
"
void Otk_Set_Window_Name( char *name );

float Otk_Set_Render_Quality_Hint( float q );

%feature("docstring") OtkMainLoop "
  This function is one of the most important.  It is the Otk event-loop.
  It makes everything happen.  You need to call this once, usually near the
  end of your application's main() routine.  It does not return
  until the application exits.  The OtkMainLoop routine is an infinite while
  loop that waits for keyboard or mouse input, then calls appropriate
  handlers, and then waits for more input.  Otk is single-threaded.  
  This function is called last, after the Otk calls which set up the initial
  window.

  See also OtkUpdateCheck().  OtkMainLoop is equivalent to a while(1) loop around OtkUpdateCheck().
"
void OtkMainLoop();
%feature("docstring") OtkDrawObjectTree "
  Displays the full list of current Otk Widgets with parent-child hierarchy.
"
void OtkDrawObjectTree();

# Gadget_lib

#define Otk_SC_Gauge1   2001
#define Otk_SC_Gauge2   2002
#define Otk_SC_LEDmeter 2100
#define Otk_SC_BarMeter 2200
#define Otk_SC_XYgraph  2300
#define Otk_SC_XYaxis   2301
#define Otk_SC_XYdata   2302
#define OTK_SC_IndicatorLight 2400

OtkWidget Otk_MakeGauge2( OtkWidget container, float left, float top, float horiz_size, float vert_size, char *title );
void Otk_SetGauge2( OtkWidget container, float value, OtkColor tmpcolor );

void Otk_SetLEDmeter( OtkWidget container, float value, OtkColor offcolor, OtkColor oncolor );
OtkWidget Otk_MakeLEDmeter( OtkWidget container, float left, float top, float horiz_size, float vert_size, int nbars, char orientation, OtkColor tmpcolor );

OtkWidget Otk_MakeBarMeter( OtkWidget container, float left, float top, float horiz_size, float vert_size, char orientation, OtkColor tmpcolor );
void Otk_SetBarMeter( OtkWidget container, float value );

OtkWidget Otk_Plot_Init( OtkWidget container, char *xlabel, char *ylabel, float xpos, float ypos, float width, float height, OtkColor fgcolor, OtkColor bgcolor );
void Otk_Plot_Set_Axes( OtkWidget plotbox, float xmin, float xmax, int xgrids,  
                                           float ymin, float ymax, int ygrids,
					   OtkColor tcolor, OtkColor gcolor );
%typemap(in) (int npoints, float *array[2]) {
  int j, size0, size1;
  PyObject *list0;
  PyObject *list1;
  $2 = malloc(sizeof(float*)*2);
  if (!PyList_Check($input)) {
    PyErr_SetString(PyExc_ValueError, "Expecting a list");
    return NULL;
  }
  $1 = PyList_Size($input);
  if ($1 < 2) {
    PyErr_SetString(PyExc_ValueError, "Expectinng a list of 2 by N elements");
    return NULL;
  }
  list0 = PyList_GetItem($input,0);
  list1 = PyList_GetItem($input,1);
  size0 = PyList_Size(list0);
  size1 = PyList_Size(list1);
  if (size0 > size1)
    $1 = size1;
  else
    $1 = size0;
  $2[0] = malloc(sizeof(float)*$1);
  $2[1] = malloc(sizeof(float)*$1);
  for (j=0;j<$1;j++) {
    /* TODO Check for correct type */
    $2[0][j] = PyFloat_AsDouble(PyList_GetItem(list0,j));
    $2[1][j] = PyFloat_AsDouble(PyList_GetItem(list1,j));
  }
}

%typemap(freearg) (int npoints, float *array[2]) {
   if ($2) {
     if ($2[0])
        free($2[0]);
     if ($2[1])
        free($2[1]);
     free($2);
   }
}
%rename(Otk_Plot_Data) my_Plot_Data;
void my_Plot_Data( OtkWidget plotbox, int npoints, float *array[2], OtkColor color);
%rename(Otk_Plot_Data_AutoScaled) my_Plot_Data_AutoScaled;
void my_Plot_Data_AutoScaled( OtkWidget plotbox, int npoints, float *array[2], OtkColor dcolor, OtkColor tcolor, OtkColor gcolor, int xgrids, int ygrids );

void Otk_StripChart_Init( OtkWidget plotbox, int xrange, float ymin, float ymax, OtkColor text_color, OtkColor grid_color, OtkColor data_color );
void Otk_StripChart_AddPoint( OtkWidget plotbox, float value );

OtkWidget Otk_AddIndicatorLight( OtkWidget container, float x, float y, float radius, OtkColor color );
void Otk_SetIndicatorLight( OtkWidget indlght, OtkColor color );
void Otk_SetIndicatorLightBorder( OtkWidget indlght, OtkColor color );

/* define standard colors */
%pythoncode %{
Otk_White  = OtkSetColor( 1.0, 1.0, 1.0 )     
Otk_LightGray  =  OtkSetColor( 0.8, 0.8, 0.8 )
Otk_Gray       =  OtkSetColor( 0.5, 0.5, 0.5 )
Otk_DarkGray   =  OtkSetColor( 0.25, 0.25, 0.25 )
Otk_Black =  OtkSetColor( 0.0, 0.0, 0.0 )
Otk_Red   =  OtkSetColor( 1.0, 0.0, 0.0 )
Otk_Green =  OtkSetColor( 0.0, 1.0, 0.0 )
Otk_Blue  =  OtkSetColor( 0.0, 0.0, 1.0 )
Otk_Cyan  =  OtkSetColor( 0.0, 1.0, 1.0 )
Otk_Yellow = OtkSetColor( 1.0, 1.0, 0.0 )
Otk_Orange = OtkSetColor( 1.0, 0.5, 0.0 )
Otk_Violet = OtkSetColor( 1.0, 0.0, 1.0 )
%}

